@@ -7,6 +7,7 @@ import pathlib
 from runpy import run_module
 import sqlite_utils
 from sqlite_utils.db import AlterError, BadMultiValues, DescIndex, NoTable
+from sqlite_utils.plugins import pm, get_plugins
 from sqlite_utils.utils import maximize_csv_field_size_limit
 from sqlite_utils import recipes
 import textwrap
@@ -33,6 +34,11 @@ from .utils import (
     TypeTracker,
 )
 
+try:
+    import trogon  # type: ignore
+except ImportError:
+    trogon = None
+
 
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
@@ -112,6 +118,10 @@ def cli():
     pass
 
 
+if trogon is not None:
+    cli = trogon.tui()(cli)
+
+
 @cli.command()
 @click.argument(
     "path",
@@ -386,7 +396,7 @@ def dump(path, load_extension):
     """
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
-    for line in db.conn.iterdump():
+    for line in db.iterdump():
         click.echo(line)
 
 
@@ -801,6 +811,7 @@ _import_options = (
     click.option("--nl", is_flag=True, help="Expect newline-delimited JSON"),
     click.option("-c", "--csv", is_flag=True, help="Expect CSV input"),
     click.option("--tsv", is_flag=True, help="Expect TSV input"),
+    click.option("--empty-null", is_flag=True, help="Treat empty strings as NULL"),
     click.option(
         "--lines",
         is_flag=True,
@@ -859,6 +870,7 @@ def insert_upsert_options(*, require_pk=False):
                 click.option(
                     "--batch-size", type=int, default=100, help="Commit every X records"
                 ),
+                click.option("--stop-after", type=int, help="Stop after X records"),
                 click.option(
                     "--alter",
                     is_flag=True,
@@ -906,6 +918,7 @@ def insert_upsert_implementation(
     nl,
     csv,
     tsv,
+    empty_null,
     lines,
     text,
     convert,
@@ -916,6 +929,7 @@ def insert_upsert_implementation(
     no_headers,
     encoding,
     batch_size,
+    stop_after,
     alter,
     upsert,
     ignore=False,
@@ -940,6 +954,8 @@ def insert_upsert_implementation(
         raise click.ClickException("Use just one of --nl, --csv or --tsv")
     if (csv or tsv) and flatten:
         raise click.ClickException("--flatten cannot be used with --csv or --tsv")
+    if empty_null and not (csv or tsv):
+        raise click.ClickException("--empty-null can only be used with --csv or --tsv")
     if encoding and not (csv or tsv):
         raise click.ClickException("--encoding must be used with --csv or --tsv")
     if pk and len(pk) == 1:
@@ -948,14 +964,15 @@ def insert_upsert_implementation(
 
     # The --sniff option needs us to buffer the file to peek ahead
     sniff_buffer = None
+    decoded_buffer = None
     if sniff:
         sniff_buffer = io.BufferedReader(file, buffer_size=4096)
-        decoded = io.TextIOWrapper(sniff_buffer, encoding=encoding)
+        decoded_buffer = io.TextIOWrapper(sniff_buffer, encoding=encoding)
     else:
-        decoded = io.TextIOWrapper(file, encoding=encoding)
+        decoded_buffer = io.TextIOWrapper(file, encoding=encoding)
 
     tracker = None
-    with file_progress(decoded, silent=silent) as decoded:
+    with file_progress(decoded_buffer, silent=silent) as decoded:
         if csv or tsv:
             if sniff:
                 # Read first 2048 bytes and use that to detect
@@ -977,7 +994,13 @@ def insert_upsert_implementation(
                 reader = itertools.chain([first_row], reader)
             else:
                 headers = first_row
-            docs = (dict(zip(headers, row)) for row in reader)
+            if empty_null:
+                docs = (
+                    dict(zip(headers, [None if cell == "" else cell for cell in row]))
+                    for row in reader
+                )
+            else:
+                docs = (dict(zip(headers, row)) for row in reader)
             if detect_types:
                 tracker = TypeTracker()
                 docs = tracker.wrap(docs)
@@ -993,13 +1016,18 @@ def insert_upsert_implementation(
                     docs = json.load(decoded)
                     if isinstance(docs, dict):
                         docs = [docs]
-            except json.decoder.JSONDecodeError:
+            except json.decoder.JSONDecodeError as ex:
                 raise click.ClickException(
-                    "Invalid JSON - use --csv for CSV or --tsv for TSV files"
+                    "Invalid JSON - use --csv for CSV or --tsv for TSV files\n\nJSON error: {}".format(
+                        ex
+                    )
                 )
             if flatten:
                 docs = (_flatten(doc) for doc in docs)
 
+    if stop_after:
+        docs = itertools.islice(docs, stop_after)
+
     if convert:
         variable = "row"
         if lines:
@@ -1079,6 +1107,12 @@ def insert_upsert_implementation(
     if tracker is not None:
         db[table].transform(types=tracker.types)
 
+    # Clean up open file-like objects
+    if sniff_buffer:
+        sniff_buffer.close()
+    if decoded_buffer:
+        decoded_buffer.close()
+
 
 def _find_variables(tb, vars):
     to_find = list(vars)
@@ -1118,6 +1152,7 @@ def insert(
     nl,
     csv,
     tsv,
+    empty_null,
     lines,
     text,
     convert,
@@ -1128,6 +1163,7 @@ def insert(
     no_headers,
     encoding,
     batch_size,
+    stop_after,
     alter,
     detect_types,
     analyze,
@@ -1193,6 +1229,7 @@ def insert(
             nl,
             csv,
             tsv,
+            empty_null,
             lines,
             text,
             convert,
@@ -1203,6 +1240,7 @@ def insert(
             no_headers,
             encoding,
             batch_size,
+            stop_after,
             alter=alter,
             upsert=False,
             ignore=ignore,
@@ -1230,11 +1268,13 @@ def upsert(
     nl,
     csv,
     tsv,
+    empty_null,
     lines,
     text,
     convert,
     imports,
     batch_size,
+    stop_after,
     delimiter,
     quotechar,
     sniff,
@@ -1271,6 +1311,7 @@ def upsert(
             nl,
             csv,
             tsv,
+            empty_null,
             lines,
             text,
             convert,
@@ -1281,6 +1322,7 @@ def upsert(
             no_headers,
             encoding,
             batch_size,
+            stop_after,
             alter=alter,
             upsert=True,
             not_null=not_null,
@@ -1318,6 +1360,7 @@ def bulk(
     nl,
     csv,
     tsv,
+    empty_null,
     lines,
     text,
     convert,
@@ -1352,6 +1395,7 @@ def bulk(
             nl=nl,
             csv=csv,
             tsv=tsv,
+            empty_null=empty_null,
             lines=lines,
             text=text,
             convert=convert,
@@ -1362,6 +1406,7 @@ def bulk(
             no_headers=no_headers,
             encoding=encoding,
             batch_size=batch_size,
+            stop_after=None,
             alter=False,
             upsert=False,
             not_null=set(),
@@ -1477,6 +1522,8 @@ def create_table(
             name text \\
             height float \\
             photo blob --pk id
+
+    Valid column types are text, integer, float and blob.
     """
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
@@ -1496,13 +1543,7 @@ def create_table(
         coltypes[name] = ctype.upper()
     # Does table already exist?
     if table in db.table_names():
-        if ignore:
-            return
-        elif replace:
-            db[table].drop()
-        elif transform:
-            pass
-        else:
+        if not ignore and not replace and not transform:
             raise click.ClickException(
                 'Table "{}" already exists. Use --replace to delete and replace it.'.format(
                     table
@@ -1514,6 +1555,8 @@ def create_table(
         not_null=not_null,
         defaults=dict(default),
         foreign_keys=fk,
+        ignore=ignore,
+        replace=replace,
         transform=transform,
     )
 
@@ -1541,6 +1584,31 @@ def duplicate(path, table, new_table, ignore, load_extension):
             raise click.ClickException('Table "{}" does not exist'.format(table))
 
 
+@cli.command(name="rename-table")
+@click.argument(
+    "path",
+    type=click.Path(file_okay=True, dir_okay=False, allow_dash=False),
+    required=True,
+)
+@click.argument("table")
+@click.argument("new_name")
+@click.option("--ignore", is_flag=True, help="If table does not exist, do nothing")
+@load_extension_option
+def rename_table(path, table, new_name, ignore, load_extension):
+    """
+    Rename this table.
+    """
+    db = sqlite_utils.Database(path)
+    _load_extensions(db, load_extension)
+    try:
+        db.rename_table(table, new_name)
+    except sqlite3.OperationalError as ex:
+        if not ignore:
+            raise click.ClickException(
+                'Table "{}" could not be renamed. {}'.format(table, str(ex))
+            )
+
+
 @cli.command(name="drop-table")
 @click.argument(
     "path",
@@ -1651,6 +1719,7 @@ def drop_view(path, view, ignore, load_extension):
 )
 @output_options
 @click.option("-r", "--raw", is_flag=True, help="Raw output, first column of first row")
+@click.option("--raw-lines", is_flag=True, help="Raw output, first column of each row")
 @click.option(
     "-p",
     "--param",
@@ -1675,6 +1744,7 @@ def query(
     fmt,
     json_cols,
     raw,
+    raw_lines,
     param,
     load_extension,
     functions,
@@ -1698,7 +1768,19 @@ def query(
         _register_functions(db, functions)
 
     _execute_query(
-        db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+        db,
+        sql,
+        param,
+        raw,
+        raw_lines,
+        table,
+        csv,
+        tsv,
+        no_headers,
+        fmt,
+        nl,
+        arrays,
+        json_cols,
     )
 
 
@@ -1726,6 +1808,7 @@ def query(
 )
 @output_options
 @click.option("-r", "--raw", is_flag=True, help="Raw output, first column of first row")
+@click.option("--raw-lines", is_flag=True, help="Raw output, first column of each row")
 @click.option(
     "-p",
     "--param",
@@ -1771,6 +1854,7 @@ def memory(
     fmt,
     json_cols,
     raw,
+    raw_lines,
     param,
     encoding,
     no_detect_types,
@@ -1814,47 +1898,50 @@ def memory(
     stem_counts = {}
     for i, path in enumerate(paths):
         # Path may have a :format suffix
+        fp = None
         if ":" in path and path.rsplit(":", 1)[-1].upper() in Format.__members__:
             path, suffix = path.rsplit(":", 1)
             format = Format[suffix.upper()]
         else:
             format = None
         if path in ("-", "stdin"):
-            csv_fp = sys.stdin.buffer
-            csv_table = "stdin"
+            fp = sys.stdin.buffer
+            file_table = "stdin"
         else:
-            csv_path = pathlib.Path(path)
-            stem = csv_path.stem
+            file_path = pathlib.Path(path)
+            stem = file_path.stem
             if stem_counts.get(stem):
-                csv_table = "{}_{}".format(stem, stem_counts[stem])
+                file_table = "{}_{}".format(stem, stem_counts[stem])
             else:
-                csv_table = stem
+                file_table = stem
             stem_counts[stem] = stem_counts.get(stem, 1) + 1
-            csv_fp = csv_path.open("rb")
-        rows, format_used = rows_from_file(csv_fp, format=format, encoding=encoding)
+            fp = file_path.open("rb")
+        rows, format_used = rows_from_file(fp, format=format, encoding=encoding)
         tracker = None
         if format_used in (Format.CSV, Format.TSV) and not no_detect_types:
             tracker = TypeTracker()
             rows = tracker.wrap(rows)
         if flatten:
             rows = (_flatten(row) for row in rows)
-        db[csv_table].insert_all(rows, alter=True)
+        db[file_table].insert_all(rows, alter=True)
         if tracker is not None:
-            db[csv_table].transform(types=tracker.types)
+            db[file_table].transform(types=tracker.types)
         # Add convenient t / t1 / t2 views
         view_names = ["t{}".format(i + 1)]
         if i == 0:
             view_names.append("t")
         for view_name in view_names:
             if not db[view_name].exists():
-                db.create_view(view_name, "select * from [{}]".format(csv_table))
+                db.create_view(view_name, "select * from [{}]".format(file_table))
+        if fp:
+            fp.close()
 
     if analyze:
         _analyze(db, tables=None, columns=None, save=False)
         return
 
     if dump:
-        for line in db.conn.iterdump():
+        for line in db.iterdump():
             click.echo(line)
         return
 
@@ -1864,7 +1951,7 @@ def memory(
 
     if save:
         db2 = sqlite_utils.Database(save)
-        for line in db.conn.iterdump():
+        for line in db.iterdump():
             db2.execute(line)
         return
 
@@ -1877,12 +1964,36 @@ def memory(
         _register_functions(db, functions)
 
     _execute_query(
-        db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+        db,
+        sql,
+        param,
+        raw,
+        raw_lines,
+        table,
+        csv,
+        tsv,
+        no_headers,
+        fmt,
+        nl,
+        arrays,
+        json_cols,
     )
 
 
 def _execute_query(
-    db, sql, param, raw, table, csv, tsv, no_headers, fmt, nl, arrays, json_cols
+    db,
+    sql,
+    param,
+    raw,
+    raw_lines,
+    table,
+    csv,
+    tsv,
+    no_headers,
+    fmt,
+    nl,
+    arrays,
+    json_cols,
 ):
     with db.conn:
         try:
@@ -1901,6 +2012,13 @@ def _execute_query(
                 sys.stdout.buffer.write(data)
             else:
                 sys.stdout.write(str(data))
+        elif raw_lines:
+            for row in cursor:
+                data = row[0]
+                if isinstance(data, bytes):
+                    sys.stdout.buffer.write(data + b"\n")
+                else:
+                    sys.stdout.write(str(data) + "\n")
         elif fmt or table:
             print(
                 tabulate.tabulate(
@@ -2578,12 +2696,20 @@ def insert_files(
     help="Specific columns to analyze",
 )
 @click.option("--save", is_flag=True, help="Save results to _analyze_tables table")
+@click.option("--common-limit", type=int, default=10, help="How many common values")
+@click.option("--no-most", is_flag=True, default=False, help="Skip most common values")
+@click.option(
+    "--no-least", is_flag=True, default=False, help="Skip least common values"
+)
 @load_extension_option
 def analyze_tables(
     path,
     tables,
     columns,
     save,
+    common_limit,
+    no_most,
+    no_least,
     load_extension,
 ):
     """Analyze the columns in one or more tables
@@ -2595,31 +2721,47 @@ def analyze_tables(
     """
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
-    _analyze(db, tables, columns, save)
+    _analyze(db, tables, columns, save, common_limit, no_most, no_least)
 
 
-def _analyze(db, tables, columns, save):
+def _analyze(db, tables, columns, save, common_limit=10, no_most=False, no_least=False):
     if not tables:
         tables = db.table_names()
     todo = []
     table_counts = {}
+    seen_columns = set()
     for table in tables:
         table_counts[table] = db[table].count
         for column in db[table].columns:
             if not columns or column.name in columns:
                 todo.append((table, column.name))
+                seen_columns.add(column.name)
+    # Check the user didn't specify a column that doesn't exist
+    if columns and (set(columns) - seen_columns):
+        raise click.ClickException(
+            "These columns were not found: {}".format(
+                ", ".join(sorted(set(columns) - seen_columns))
+            )
+        )
     # Now we now how many we need to do
     for i, (table, column) in enumerate(todo):
         column_details = db[table].analyze_column(
-            column, total_rows=table_counts[table], value_truncate=80
+            column,
+            common_limit=common_limit,
+            total_rows=table_counts[table],
+            value_truncate=80,
+            most_common=not no_most,
+            least_common=not no_least,
         )
         if save:
             db["_analyze_tables_"].insert(
                 column_details._asdict(), pk=("table", "column"), replace=True
             )
-        most_common_rendered = _render_common(
-            "\n\n  Most common:", column_details.most_common
-        )
+        most_common_rendered = ""
+        if column_details.num_null != column_details.total_rows:
+            most_common_rendered = _render_common(
+                "\n\n  Most common:", column_details.most_common
+            )
         least_common_rendered = _render_common(
             "\n\n  Least common:", column_details.least_common
         )
@@ -2651,15 +2793,23 @@ def _analyze(db, tables, columns, save):
 
 
 @cli.command()
-@click.argument("packages", nargs=-1, required=True)
+@click.argument("packages", nargs=-1, required=False)
 @click.option(
     "-U", "--upgrade", is_flag=True, help="Upgrade packages to latest version"
 )
-def install(packages, upgrade):
+@click.option(
+    "-e",
+    "--editable",
+    type=click.Path(readable=True, exists=True, dir_okay=True, file_okay=False),
+    help="Install a project in editable mode from this path",
+)
+def install(packages, upgrade, editable):
     """Install packages from PyPI into the same environment as sqlite-utils"""
     args = ["pip", "install"]
     if upgrade:
         args += ["--upgrade"]
+    if editable:
+        args += ["--editable", str(editable)]
     args += list(packages)
     sys.argv = args
     run_module("pip", run_name="__main__")
@@ -2750,6 +2900,7 @@ def _generate_convert_help():
     type=click.Choice(["integer", "float", "blob", "text"]),
 )
 @click.option("--drop", is_flag=True, help="Drop original column afterwards")
+@click.option("--no-skip-false", is_flag=True, help="Don't skip falsey values")
 @click.option("-s", "--silent", is_flag=True, help="Don't show a progress bar")
 def convert(
     db_path,
@@ -2764,6 +2915,7 @@ def convert(
     output,
     output_type,
     drop,
+    no_skip_false,
     silent,
 ):
     sqlite3.enable_callback_tracebacks(True)
@@ -2785,11 +2937,16 @@ def convert(
         raise click.ClickException(str(e))
     if dry_run:
         # Pull first 20 values for first column and preview them
-        preview = lambda v: fn(v) if v else v  # noqa: E731
         if multi:
-            preview = (
-                lambda v: json.dumps(fn(v), default=repr) if v else v
-            )  # noqa: E731
+
+            def preview(v):
+                return json.dumps(fn(v), default=repr) if v else v
+
+        else:
+
+            def preview(v):
+                return fn(v) if v else v
+
         db.conn.create_function("preview_transform", 1, preview)
         sql = """
             select
@@ -2821,6 +2978,7 @@ def convert(
                 output=output,
                 output_type=output_type,
                 drop=drop,
+                skip_false=not no_skip_false,
                 multi=multi,
                 show_progress=not silent,
             )
@@ -2942,6 +3100,15 @@ def create_spatial_index(db_path, table, column_name, load_extension):
     db[table].create_spatial_index(column_name)
 
 
+@cli.command(name="plugins")
+def plugins_list():
+    "List installed plugins"
+    click.echo(json.dumps(get_plugins(), indent=2))
+
+
+pm.hook.register_commands(cli=cli)
+
+
 def _render_common(title, values):
     if values is None:
         return ""
