@@ -14,20 +14,28 @@ from typing import Dict, cast, BinaryIO, Iterable, Optional, Tuple, Type
 import click
 
 try:
-    import pysqlite3 as sqlite3  # type: ignore
-    import pysqlite3.dbapi2  # type: ignore
+    import pysqlite3 as sqlite3  # noqa: F401
+    from pysqlite3 import dbapi2  # noqa: F401
 
-    OperationalError = pysqlite3.dbapi2.OperationalError
+    OperationalError = dbapi2.OperationalError
 except ImportError:
-    # https://github.com/python/mypy/issues/1153#issuecomment-253842414
-    import sqlite3  # type: ignore
+    try:
+        import sqlean as sqlite3  # noqa: F401
+        from sqlean import dbapi2  # noqa: F401
 
-    OperationalError = sqlite3.OperationalError
+        OperationalError = dbapi2.OperationalError
+    except ImportError:
+        import sqlite3  # noqa: F401
+        from sqlite3 import dbapi2  # noqa: F401
+
+        OperationalError = dbapi2.OperationalError
 
 
 SPATIALITE_PATHS = (
     "/usr/lib/x86_64-linux-gnu/mod_spatialite.so",
     "/usr/local/lib/mod_spatialite.dylib",
+    "/usr/local/lib/mod_spatialite.so",
+    "/opt/homebrew/lib/mod_spatialite.dylib",
 )
 
 # Mainly so we can restore it if needed in the tests:
@@ -302,7 +310,13 @@ def rows_from_file(
     elif format is None:
         # Detect the format, then call this recursively
         buffered = io.BufferedReader(cast(io.RawIOBase, fp), buffer_size=4096)
-        first_bytes = buffered.peek(2048).strip()
+        try:
+            first_bytes = buffered.peek(2048).strip()
+        except AttributeError:
+            # Likely the user passed a TextIO when this needs a BytesIO
+            raise TypeError(
+                "rows_from_file() requires a file-like object that supports peek(), such as io.BytesIO"
+            )
         if first_bytes.startswith(b"[") or first_bytes.startswith(b"{"):
             # TODO: Detect newline-JSON
             return rows_from_file(buffered, format=Format.JSON)
