@@ -10,7 +10,8 @@ import time
 def test_insert_simple(tmpdir):
     json_path = str(tmpdir / "dog.json")
     db_path = str(tmpdir / "dogs.db")
-    open(json_path, "w").write(json.dumps({"name": "Cleo", "age": 4}))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps({"name": "Cleo", "age": 4}))
     result = CliRunner().invoke(cli.cli, ["insert", db_path, "dogs", json_path])
     assert 0 == result.exit_code
     assert [{"age": 4, "name": "Cleo"}] == list(
@@ -42,9 +43,9 @@ def test_insert_invalid_json_error(tmpdir):
         input="name,age\nCleo,4",
     )
     assert result.exit_code == 1
-    assert (
-        result.output
-        == "Error: Invalid JSON - use --csv for CSV or --tsv for TSV files\n"
+    assert result.output == (
+        "Error: Invalid JSON - use --csv for CSV or --tsv for TSV files\n\n"
+        "JSON error: Expecting value: line 1 column 1 (char 0)\n"
     )
 
 
@@ -78,7 +79,8 @@ def test_insert_json_flatten_nl(tmpdir):
 
 def test_insert_with_primary_key(db_path, tmpdir):
     json_path = str(tmpdir / "dog.json")
-    open(json_path, "w").write(json.dumps({"id": 1, "name": "Cleo", "age": 4}))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps({"id": 1, "name": "Cleo", "age": 4}))
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "dogs", json_path, "--pk", "id"]
     )
@@ -93,7 +95,8 @@ def test_insert_with_primary_key(db_path, tmpdir):
 def test_insert_multiple_with_primary_key(db_path, tmpdir):
     json_path = str(tmpdir / "dogs.json")
     dogs = [{"id": i, "name": "Cleo {}".format(i), "age": i + 3} for i in range(1, 21)]
-    open(json_path, "w").write(json.dumps(dogs))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps(dogs))
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "dogs", json_path, "--pk", "id"]
     )
@@ -109,7 +112,8 @@ def test_insert_multiple_with_compound_primary_key(db_path, tmpdir):
         {"breed": "mixed", "id": i, "name": "Cleo {}".format(i), "age": i + 3}
         for i in range(1, 21)
     ]
-    open(json_path, "w").write(json.dumps(dogs))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps(dogs))
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "dogs", json_path, "--pk", "id", "--pk", "breed"]
     )
@@ -134,7 +138,8 @@ def test_insert_not_null_default(db_path, tmpdir):
         {"id": i, "name": "Cleo {}".format(i), "age": i + 3, "score": 10}
         for i in range(1, 21)
     ]
-    open(json_path, "w").write(json.dumps(dogs))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps(dogs))
     result = CliRunner().invoke(
         cli.cli,
         ["insert", db_path, "dogs", json_path, "--pk", "id"]
@@ -182,7 +187,8 @@ def test_insert_ignore(db_path, tmpdir):
     db = Database(db_path)
     db["dogs"].insert({"id": 1, "name": "Cleo"}, pk="id")
     json_path = str(tmpdir / "dogs.json")
-    open(json_path, "w").write(json.dumps([{"id": 1, "name": "Bailey"}]))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps([{"id": 1, "name": "Bailey"}]))
     # Should raise error without --ignore
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "dogs", json_path, "--pk", "id"]
@@ -212,7 +218,8 @@ def test_insert_ignore(db_path, tmpdir):
 def test_insert_csv_tsv(content, options, db_path, tmpdir):
     db = Database(db_path)
     file_path = str(tmpdir / "insert.csv-tsv")
-    open(file_path, "w").write(content)
+    with open(file_path, "w") as fp:
+        fp.write(content)
     result = CliRunner().invoke(
         cli.cli,
         ["insert", db_path, "data", file_path] + options,
@@ -222,6 +229,49 @@ def test_insert_csv_tsv(content, options, db_path, tmpdir):
     assert [{"foo": "1", "bar": "2", "baz": "cat,dog"}] == list(db["data"].rows)
 
 
+@pytest.mark.parametrize("empty_null", (True, False))
+def test_insert_csv_empty_null(db_path, empty_null):
+    options = ["--csv"]
+    if empty_null:
+        options.append("--empty-null")
+    result = CliRunner().invoke(
+        cli.cli,
+        ["insert", db_path, "data", "-"] + options,
+        catch_exceptions=False,
+        input="foo,bar,baz\n1,,cat,dog",
+    )
+    assert result.exit_code == 0
+    db = Database(db_path)
+    assert [r for r in db["data"].rows] == [
+        {"foo": "1", "bar": None if empty_null else "", "baz": "cat"}
+    ]
+
+
+@pytest.mark.parametrize(
+    "input,args",
+    (
+        (
+            json.dumps(
+                [{"name": "One"}, {"name": "Two"}, {"name": "Three"}, {"name": "Four"}]
+            ),
+            [],
+        ),
+        ("name\nOne\nTwo\nThree\nFour\n", ["--csv"]),
+    ),
+)
+def test_insert_stop_after(tmpdir, input, args):
+    db_path = str(tmpdir / "data.db")
+    result = CliRunner().invoke(
+        cli.cli,
+        ["insert", db_path, "rows", "-", "--stop-after", "2"] + args,
+        input=input,
+    )
+    assert 0 == result.exit_code
+    assert [{"name": "One"}, {"name": "Two"}] == list(
+        Database(db_path).query("select * from rows")
+    )
+
+
 @pytest.mark.parametrize(
     "options",
     (
@@ -233,7 +283,8 @@ def test_insert_csv_tsv(content, options, db_path, tmpdir):
 )
 def test_only_allow_one_of_nl_tsv_csv(options, db_path, tmpdir):
     file_path = str(tmpdir / "insert.csv-tsv")
-    open(file_path, "w").write("foo")
+    with open(file_path, "w") as fp:
+        fp.write("foo")
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "data", file_path] + options
     )
@@ -251,7 +302,8 @@ def test_insert_replace(db_path, tmpdir):
         {"id": 2, "name": "Insert replaced 2", "age": 4},
         {"id": 21, "name": "Fresh insert 21", "age": 6},
     ]
-    open(json_path, "w").write(json.dumps(insert_replace_dogs))
+    with open(json_path, "w") as fp:
+        fp.write(json.dumps(insert_replace_dogs))
     result = CliRunner().invoke(
         cli.cli, ["insert", db_path, "dogs", json_path, "--pk", "id", "--replace"]
     )
